/*****************************************************************************
 * Copyright (c) 2014 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.
 *
 *    NAME: r00_post_mtm_6_crd4.sas
 *
 * PURPOSE: Check for granularity criteria for regulatory retail portfolio. 
 *          Check for SME capital deduction qualifying criteria.
 *   USAGE: 
 *
 *   NOTES: 5/23/2014
 *          Updated retail granularity criteria based on CRR regulations.
 *          Added SME capital deduction qualifying criteria.
 *          More qualifying criteria can be added here as needed.
 *          
 *
 *****************************************************************************/

%macro r00_post_mtm_6_crd4;

    %if APPROACH = 'BASEL_I' %then %goto EXIT;

/*- qualifying criteria for regulatory retail portfolio -*/
    %local TOTAL_RETAIL_EXPOSURE_STD TOTAL_RETAIL_EXPOSURE_IRB;
    %let TOTAL_RETAIL_EXPOSURE_STD  = 0;
    %let TOTAL_RETAIL_EXPOSURE_IRB  = 0;

/*- 1: get all non-sec-underlying retail exposures -*/
    data _all_retail_group_(keep=EXP_ID EAD COUNTERPARTY_RK COUNTERPARTY_GROUP APPROACH  APPROACH_CLASS COUNTERPARTY_CLASS COUNTERPARTY_SUBCLASS
                           EXT_ISSUE_MAP_RATING SHORT_TERM_ASSESS_FLG STD_RW_BUCKET EXT_MAP_RATING
                           STD_PRODUCT_CLASS STD_PRODUCT_SUBCLASS IRB_PRODUCT_CLASS IRB_PRODUCT_SUBCLASS
                           PAST_DUE_CLAIM_FLG IN_DEFAULT_FLG
                            );
                  
       set &RESLIB1..all_exposures ;
       /*- add corporate class in the retail data query because some retail counterparty_groups -*/
       /*- can have both corporate and retail counterparties in it.                             -*/
       where ((COUNTERPARTY_CLASS = 'RETAIL') or (COUNTERPARTY_CLASS = 'CORPORATE' and not missing(COUNTERPARTY_GROUP)))  
             and (SEC_UNDERLYING_EXP_FLG NE 1)
             and ((APPROACH='STD' and STD_PRODUCT_CLASS not in ('SEC', 'SECURITIZATION')) or
                  (APPROACH='IRB' and IRB_PRODUCT_CLASS not in ('SEC', 'SECURITIZATION')));
    run ;
     
    data _corp_group_ (index=(COUNTERPARTY_GROUP))
         _all_retail_  ;
       set _all_retail_group_ ;
       
       if COUNTERPARTY_CLASS ne 'RETAIL' then output _corp_group_ ;       
       else output _all_retail_ ;
    run ;
    
    proc sql noprint ;
       create table _retail_group_id_ as
       select distinct COUNTERPARTY_GROUP
       from _all_retail_ 
       order by COUNTERPARTY_GROUP
       ;
    quit ;
       
    data _mixed_group_ ;
       merge _corp_group_ (in=a)
             _retail_group_id_(in=b) ; 
       by COUNTERPARTY_GROUP ;

       if a and b then output ;
    run ;
    
    proc append data=_mixed_group_ base=_all_retail_ force ;run ;
    
    data _all_retail_ ;
       set _all_retail_  end=last;
    
       length CPTY_GROUP_ID $32. ;   
    
    /*- retail granularity check is based on obligor client or group of connected clients: CRR Article 123(c) -*/
        if compress(COUNTERPARTY_GROUP) ne '' then CPTY_GROUP_ID = COUNTERPARTY_GROUP;
        else CPTY_GROUP_ID= trim(left(put(COUNTERPARTY_RK, best.))) ; 
        
        retain TOTAL_RETAIL_EXPOSURE_STD TOTAL_RETAIL_EXPOSURE_IRB 0;
        
        /*- total exposure for STD regulatory portfolio -*/    
            if APPROACH = 'STD' and not (STD_PRODUCT_CLASS in ('MORTGAGE', 'LEASE_IMM_PROPERTY') and STD_PRODUCT_SUBCLASS ='RESIDENTIAL')
            then
                TOTAL_RETAIL_EXPOSURE_STD = sum(TOTAL_RETAIL_EXPOSURE_STD, max(EAD,0));
        /*- total exposure for IRB regulatory portfolio -*/
            else if APPROACH = 'IRB' then
                TOTAL_RETAIL_EXPOSURE_IRB = sum(TOTAL_RETAIL_EXPOSURE_IRB, max(EAD,0));
    
        if last then do;
            /*- create macro variable for total exposure for regulatory retail portfolio -*/
            /*- under STANDARDIZED approach: Basel II - 70 -*/    
            call symput('TOTAL_RETAIL_EXPOSURE_STD', TOTAL_RETAIL_EXPOSURE_STD);
            /*- create macro variable for total exposure for regulatory retail portfolio -*/
            /*- under IRB approach: Basel II - 231 - 234 -*/        
            call symput('TOTAL_RETAIL_EXPOSURE_IRB', TOTAL_RETAIL_EXPOSURE_IRB);
        end;
     run;    


   /**************************************************************************************************/
   /* get unsecured EAD for exposures secured by residiential immovable property under STD approach. */
   /* CRR Article 123(c): Only exposures fully and completely secured on residential property        */  
   /* collateral should be excluded for retail granularity check.                                    */
   /**************************************************************************************************/
   
    /*- get all residential mortgage/lease exposures and the residential property collaterals -*/
    /*- associated with them.                                                                 -*/                                                            
    proc sql noprint;
       create table _retail_std_exp_imp_1 as
       select exp.EXP_ID, exp.APPROACH, exp.EAD, exp.STD_PRODUCT_CLASS,  exp.STD_PRODUCT_SUBCLASS,
       crm.CRM_ID, crm.CRM_TYPE,crm.CRM_SUBTYPE, crm.CRM_VALUE, link.EXPOSURE_RANK, link.CRM_RANK 
       from _all_retail_ as exp
       inner join RD_POS.exposure_crm_link as link 
       on exp.EXP_ID = link.EXP_ID
       inner join &RESLIB1..all_crms as crm 
       on crm.CRM_ID = link.CRM_ID

       where exp.APPROACH = 'STD' and exp.STD_PRODUCT_CLASS in ('MORTGAGE','LEASE_IMM_PROPERTY') 
             and exp.STD_PRODUCT_SUBCLASS = 'RESIDENTIAL' 
             and crm.CRM_TYPE = 'STD_PHYSICAL_COLLATERAL' and crm.CRM_SUBTYPE = 'RRE'
      ;
    quit;   

    
    proc sort data= _retail_std_exp_imp_1 nodupkey;
    by CRM_ID EXP_ID;
    run;
   
   /*- separate exposures into two subsets       -*/
   /*- 1) a CRM is linked to one exposure only   -*/
   /*- 2) a CRM is linked to multiple exposures  -*/
    data _retail_std_exp_imp_A 
         _retail_std_exp_imp_1_2 ;
       set _retail_std_exp_imp_1;
       by CRM_ID ;
       if first.CRM_ID and last.CRM_ID then output _retail_std_exp_imp_A;
       else  output _retail_std_exp_imp_1_2;
    run; 

    proc sql noprint;    
       create table _retail_std_exp_id_1_2 as
       select distinct EXP_ID 
       from _retail_std_exp_imp_1_2
       order by EXP_ID 
       ;
    quit;
    
    proc sort data=_retail_std_exp_imp_A;
       by EXP_ID;
    run;

    /*- move exposures that are present in both subsets to subset 2) only */ 
    data _retail_std_exp_imp_1_1 _retail_std_exp_imp_B;
      merge _retail_std_exp_imp_A(in=A) _retail_std_exp_id_1_2(in=B);
      by EXP_ID;
      if (A and B) then output _retail_std_exp_imp_B;
      else if A then output _retail_std_exp_imp_1_1;
    run;
     
     proc append data=_retail_std_exp_imp_B base=_retail_std_exp_imp_1_2 force;
     run;
     
  
     /*************************************************/
     /* process case 1) exposures                     */
     /*************************************************/
     
     /*- define order of crm allocation for case 1) exposures -*/
      proc sort data=_retail_std_exp_imp_1_1;
        %if &MITIGANTS_ALLOCATION_METHOD = BY_RANK %then %do ;
             by EXP_ID CRM_RANK;
         %end ;
          %else %if &MITIGANTS_ALLOCATION_METHOD = OPTIMAL %then %do ;
             by EXP_ID descending CRM_VALUE;
         %end ;     
      run;
      
      /*- calculate EAD_UNSEC for case 1) exposures -*/
      data _retail_std_exp_imp_1_1;
         set _retail_std_exp_imp_1_1 ;
         by EXP_ID;
         retain EAD_SEC EAD_UNSEC ;
         if first.EXP_ID then do;                              
            EAD_UNSEC=EAD ;
            EAD_SEC = 0 ;
         end;

         if CRM_VALUE>=EAD_UNSEC then do;
            EAD_SEC = EAD_SEC + EAD_UNSEC;
            EAD_UNSEC = 0;
         end;
         else do;
            EAD_SEC =EAD_SEC + CRM_VALUE;
            EAD_UNSEC=EAD_UNSEC - CRM_VALUE;
         end;
         if last.EXP_ID then output;
      run;
    
    
    /*************************************************/
    /* process case 2) exposures                     */
    /*************************************************/
    
    /*- create an initial CRM remaining value table -*/
    data _crm_rem_value_;       
       set _retail_std_exp_imp_1_2 (keep=CRM_ID CRM_VALUE);
       CRM_VALUE_REM=CRM_VALUE;
    run;
    
    proc sort data=_crm_rem_value_ nodupkey;
       by CRM_ID;
    run;
    
    proc sql noprint;
    create index CRM_ID on _crm_rem_value_ (CRM_ID);
    quit;

    /*- define order of crm allocation for case 2)exposures -*/
    proc sort data=_retail_std_exp_imp_1_2;
       %if &MITIGANTS_ALLOCATION_METHOD = BY_RANK %then %do ;
        by EXPOSURE_RANK EXP_ID  CRM_RANK ;
       %end ;

       %else %if &MITIGANTS_ALLOCATION_METHOD = OPTIMAL %then %do ;
        by EXP_ID descending CRM_VALUE;
       %end ;
    run;   

    /*- calculate EAD_UNSEC for case 2) exposures -*/
    data _retail_std_exp_imp_ (drop = CRM_VALUE_REM rc) ;
       set _retail_std_exp_imp_1_2;
       %if &MITIGANTS_ALLOCATION_METHOD = BY_RANK %then %do ;
        by EXPOSURE_RANK EXP_ID CRM_RANK ;
       %end ;

       %else %if &MITIGANTS_ALLOCATION_METHOD = OPTIMAL %then %do ;
        by EXP_ID descending CRM_VALUE;
       %end ;

       retain EAD_SEC EAD_UNSEC ;
      
      if (_N_=1) then do;
         /*- declare a hash table for a list of CRM IDs and CRM Values -*/
          declare hash crm_hash(dataset: "_crm_rem_value_", hashexp: 20); 
           rc=crm_hash.definekey('CRM_ID');
           rc=crm_hash.definedata('CRM_VALUE_REM');
           rc=crm_hash.definedone();
           call missing(CRM_VALUE_REM);
           rc=0;
      end;

      if first.EXP_ID then do;
         EAD_UNSEC=EAD ;
         EAD_SEC = 0 ;
      end;

      /*- get CRM_VALUE_REM from the hash table -*/
      rc = crm_hash.find();
      if (rc = 0) then do;

         if CRM_VALUE_REM>=EAD_UNSEC then do;
            EAD_SEC = EAD_SEC + EAD_UNSEC;
            CRM_VALUE_REM = CRM_VALUE_REM - EAD_UNSEC;
            EAD_UNSEC = 0;
         end;
         else do;
            EAD_SEC =EAD_SEC + CRM_VALUE_REM;
            EAD_UNSEC=EAD_UNSEC - CRM_VALUE_REM;
            CRM_VALUE_REM =0;   
         end;
          /*- update and replace hash data -*/
          rc = crm_hash.replace();
      end;  
     if last.EXP_ID then output;
    run;


   /*- combine part 1) & part 2) -*/
   proc append data=_retail_std_exp_imp_1_1 base=_retail_std_exp_imp_ force;
   run;

   proc sort data=_all_retail_ nodupkey;
      by EXP_ID ;
   run ;

   proc sort data=_retail_std_exp_imp_ nodupkey;
      by EXP_ID ;
   run ;

   data _all_retail_ ;
      merge _all_retail_  _retail_std_exp_imp_(keep=EXP_ID EAD_UNSEC CRM_SUBTYPE) ; 
      by EXP_ID ;
   run ;

/*****************************************************************************************/
   

/*- 2: get retail portfolio size under STD and IRB approaches for each CPTY_GROUP_ID -*/
/*-    count number of defaulted exposures for each COUNTERPARTY_RK                  -*/         
    proc sort data=_all_retail_;
        by CPTY_GROUP_ID COUNTERPARTY_RK;
    run ;
    
    data cpty_retail(keep=CPTY_GROUP_ID TOTAL_RETAIL_CPTY_STD TOTAL_RETAIL_CPTY_IRB TOTAL_RETAIL_REVOLV_CPTY_STD TOTAL_RETAIL_REVOLV_CPTY_IRB) 
         cpty_rk_retail_std(keep=CPTY_GROUP_ID COUNTERPARTY_RK NUM_DEFAULTED_EXP_CPTY_STD)
         cpty_rk_retail_irb(keep=CPTY_GROUP_ID COUNTERPARTY_RK NUM_DEFAULTED_EXP_CPTY_IRB);
    set _all_retail_(keep=EAD EAD_UNSEC CPTY_GROUP_ID COUNTERPARTY_RK APPROACH STD_PRODUCT_CLASS STD_PRODUCT_SUBCLASS IRB_PRODUCT_CLASS 
              PAST_DUE_CLAIM_FLG IN_DEFAULT_FLG IRB_PRODUCT_SUBCLASS CRM_SUBTYPE);
    by CPTY_GROUP_ID COUNTERPARTY_RK;
    
    retain TOTAL_RETAIL_CPTY_STD TOTAL_RETAIL_CPTY_IRB TOTAL_RETAIL_REVOLV_CPTY_STD TOTAL_RETAIL_REVOLV_CPTY_IRB 
           NUM_DEFAULTED_EXP_CPTY_STD NUM_DEFAULTED_EXP_CPTY_IRB 0;

    if first.CPTY_GROUP_ID then do;
        TOTAL_RETAIL_CPTY_STD = 0;
        TOTAL_RETAIL_CPTY_IRB = 0;
        TOTAL_RETAIL_REVOLV_CPTY_STD = 0;
        TOTAL_RETAIL_REVOLV_CPTY_IRB = 0;
        if first.COUNTERPARTY_RK then do ;
           NUM_DEFAULTED_EXP_CPTY_STD = 0 ; 
           NUM_DEFAULTED_EXP_CPTY_IRB = 0 ;
        end ;   
    end;

    if APPROACH = 'STD' then do;
        /*- total retail exposure under STD for every CPTY_GROUP_ID  -*/
        if not (STD_PRODUCT_CLASS in ('MORTGAGE','LEASE_IMM_PROPERTY') and STD_PRODUCT_SUBCLASS ='RESIDENTIAL' and 
                CRM_TYPE = 'STD_PHYSICAL_COLLATERAL' and CRM_SUBTYPE = 'RRE')
           then TOTAL_RETAIL_CPTY_STD = sum(TOTAL_RETAIL_CPTY_STD, EAD);
        else if (STD_PRODUCT_CLASS in ('MORTGAGE','LEASE_IMM_PROPERTY') and STD_PRODUCT_SUBCLASS ='RESIDENTIAL' and 
                 CRM_TYPE = 'STD_PHYSICAL_COLLATERAL' and CRM_SUBTYPE = 'RRE') 
           then TOTAL_RETAIL_CPTY_STD = sum(TOTAL_RETAIL_CPTY_STD, EAD_UNSEC);
        /*- total retail revolving exposure under STD for every CPTY_GROUP_ID  -*/ 
        if STD_PRODUCT_CLASS = 'REVOLVING' then
            TOTAL_RETAIL_REVOLV_CPTY_STD = sum(TOTAL_RETAIL_REVOLV_CPTY_STD, EAD);
       
        if IN_DEFAULT_FLG = 1 or PAST_DUE_CLAIM_FLG = 1 then NUM_DEFAULTED_EXP_CPTY_STD +1 ; 
        
        if last.COUNTERPARTY_RK then do ;    
               output cpty_rk_retail_std ;
               NUM_DEFAULTED_EXP_CPTY_STD = 0;
        end ;
    end;
        
    else do;
        /*- total retail exposure under IRB for every CPTY_GROUP_ID  -*/
        /* residential real estate are excluded from the total: B2 231 and CRD art.86-4(a) */
        if not (IRB_PRODUCT_CLASS = 'MORTGAGE' and IRB_PRODUCT_SUBCLASS = 'RESIDENTIAL') then
        TOTAL_RETAIL_CPTY_IRB = sum(TOTAL_RETAIL_CPTY_IRB, EAD);
        /*- total retail revolving exposure under IRB for every CPTY_GROUP_ID  -*/
        if IRB_PRODUCT_CLASS = 'REVOLVING' then
            TOTAL_RETAIL_REVOLV_CPTY_IRB = sum(TOTAL_RETAIL_REVOLV_CPTY_IRB, EAD);
        
        if IN_DEFAULT_FLG = 1 or PAST_DUE_CLAIM_FLG = 1 then NUM_DEFAULTED_EXP_CPTY_IRB +1 ;
        
        if last.COUNTERPARTY_RK then do ;    
              output cpty_rk_retail_irb ;
              NUM_DEFAULTED_EXP_CPTY_IRB = 0;
        end ;
    end;

    

    if last.CPTY_GROUP_ID then output cpty_retail;
    run;
    
/*- 3: retail portfolio qualifying criteria check -*/    
    /*- update to get count of defaulted exposures per counterparty -*/
    data _all_retail_ ;
       merge _all_retail_  
             cpty_rk_retail_std
             cpty_rk_retail_irb ;
       by CPTY_GROUP_ID COUNTERPARTY_RK ;
    run ;   

    data &RESLIB1..retail_granularity_exception
        (where=(CPTY_CLASS_CHANGED_FLG = 1 or IRB_PRODUCT_CLASS_CHANGED_FLG = 1)
         keep=  COUNTERPARTY_RK EXP_ID CPTY_GROUP_ID APPROACH COUNTERPARTY_CLASS COUNTERPARTY_SUBCLASS EXT_ISSUE_MAP_RATING EXT_MAP_RATING
               STD_RW_BUCKET STD_RW_BUCKET_COUNTERPARTY IRB_PRODUCT_CLASS EAD TOTAL_RETAIL_EXPOSURE_STD TOTAL_RETAIL_EXPOSURE_IRB 
               TOTAL_RETAIL_CPTY_STD TOTAL_RETAIL_CPTY_IRB TOTAL_RETAIL_REVOLV_CPTY_IRB NUM_DEFAULTED_EXP_CPTY_STD 
               NUM_DEFAULTED_EXP_CPTY_IRB CPTY_CLASS_CHANGED_FLG IRB_PRODUCT_CLASS_CHANGED_FLG IN_DEFAULT_FLG APPROACH_CLASS
         index=(EXP_ID)
        );  

retain EXP_ID CPTY_GROUP_ID APPROACH COUNTERPARTY_CLASS COUNTERPARTY_SUBCLASS EXT_ISSUE_MAP_RATING EXT_MAP_RATING
       STD_RW_BUCKET STD_RW_BUCKET_COUNTERPARTY IRB_PRODUCT_CLASS EAD TOTAL_RETAIL_EXPOSURE_STD TOTAL_RETAIL_EXPOSURE_IRB 
       TOTAL_RETAIL_CPTY_STD TOTAL_RETAIL_CPTY_IRB TOTAL_RETAIL_REVOLV_CPTY_IRB NUM_DEFAULTED_EXP_CPTY_STD 
       NUM_DEFAULTED_EXP_CPTY_IRB CPTY_CLASS_CHANGED_FLG IRB_PRODUCT_CLASS_CHANGED_FLG IN_DEFAULT_FLG APPROACH_CLASS ;    

label TOTAL_RETAIL_EXPOSURE_STD="Total Exp Amt under STD for checking granularity criteria"
         TOTAL_RETAIL_EXPOSURE_IRB=" Total Exp Amt under ITB for checking granularity criteria"
         TOTAL_RETAIL_CPTY_STD="Total Exp Amt under STD for every CPTY_GROUP_ID" 
         TOTAL_RETAIL_CPTY_IRB="Total Exp Amt under IRB for every CPTY_GROUP_ID"
         TOTAL_RETAIL_REVOLV_CPTY_IRB="Total retail revolving Exp Amt under IRB for every CPTY_GROUP_ID" 
         TOTAL_RETAIL_REVOLV_CPTY_STD="Total retail revolving Exp Amt under STD for every CPTY_GROUP_ID" 
         NUM_DEFAULTED_EXP_CPTY_STD= "No. of defaulted exposures for each COUNTERPARTY under STD "  
      NUM_DEFAULTED_EXP_CPTY_IRB= "No. of defaulted exposures for each COUNTERPARTY under IRB " ; 
    merge _all_retail_ cpty_retail  ;
    by CPTY_GROUP_ID;
    
    CPTY_CLASS_CHANGED_FLG = 0;
    IRB_PRODUCT_CLASS_CHANGED_FLG = 0;
 
/*- qualifying criteria for regulatory retail portfolio under STD approach -*/
%if "&USE_UNDRAWN_RETAIL_GRAN_CHECK" = "YES" %then %do ;
    if FACILITY_FLG ne 1 ;
%end ;

    if APPROACH = 'STD' then do;
   
            if (TOTAL_RETAIL_CPTY_STD > &STD_RETAIL_CPTY_ABS_LIMIT) then do; 
               
                if COUNTERPARTY_CLASS = 'RETAIL' then do; 
                   COUNTERPARTY_CLASS = 'CORPORATE';
                   APPROACH_CLASS = 'CORPORATE';
                end;
                if COUNTERPARTY_SUBCLASS = 'RETAIL' then COUNTERPARTY_SUBCLASS = 'CORPORATE';
                
                CPTY_CLASS_CHANGED_FLG = 1;                
                STD_RW_BUCKET_COUNTERPARTY = put(EXT_MAP_RATING, $MAP_STD_RW_BUCKET_CORPORATE.);
                
                /*- update for in default status: for corporate exposures, if one exposure is defaulted,   -*/
                /*- then all other exp of the counterparty are considered defaulted: CRR Article 178       -*/
                if NUM_DEFAULTED_EXP_CPTY_STD >0 then IN_DEFAULT_FLG = 1 ; 
                
                /*- only non past dues are assigned a new rw_bucket as bucket for past dues assigned below -*/
                if not ((PAST_DUE_CLAIM_FLG = 1 or IN_DEFAULT_FLG = 1) or STD_PRODUCT_CLASS in ('OTHER_ASSETS'))
                then do; 
                        if SHORT_TERM_ASSESS_FLG = 1 then
                           STD_RW_BUCKET = put(EXT_ISSUE_MAP_RATING, $MAP_STD_RW_BUCKET_SHORT.);
                        else
                           STD_RW_BUCKET = put(EXT_ISSUE_MAP_RATING, $MAP_STD_RW_BUCKET_CORPORATE.);
                end;
            end;
     
    end;
     
   else if APPROACH ne 'STD' then do; 
/*- qualifying criteria for regulatory retail portfolio under IRB approach -*/        
        if COUNTERPARTY_SUBCLASS = 'SME' then do;
            if TOTAL_RETAIL_CPTY_IRB >= &IRB_RETAIL_SME_CPTY_ABS_LIMIT then do;
                COUNTERPARTY_CLASS = 'CORPORATE';
                CPTY_CLASS_CHANGED_FLG = 1;
                /*- change approach if COUNTERPARTY_CLASS is changed to CORPORATE -*/
                APPROACH = 'IRBA';
            end;
        end;
/*- qualifying criteria for regulatory retail revolving portfolio under IRB approach -*/        
        else if IRB_PRODUCT_CLASS = 'REVOLVING' then do;
            if TOTAL_RETAIL_REVOLV_CPTY_IRB > &IRB_RETAIL_QRRE_CPTY_ABS_LIMIT then do;
                IRB_PRODUCT_CLASS = 'OTHER';        
                IRB_PRODUCT_CLASS_CHANGED_FLG = 1;
            end;
        end;
    end;

run;

    proc sort data=rd_pos.counterparty out=&RESLIB1..counterparty;
    by CPTY_RK;
    run;

/*- 4: update for these retail exposures -*/

/*- sort the data set before the merge if netted repo-style or -*/
/*- off-balance sheet exposures have been added -*/

  %if &N_REPO_NET > 0 or &N_OBSI_NET > 0 %then %do ;
        proc sort data = &RESLIB1..all_exposures ;
            by EXP_ID ;
    run ;
    %end ; 

    %local NCHANGED;
    %let NCHANGED = %attrn(&RESLIB1..retail_granularity_exception, NOBS);
    %if &NCHANGED > 0 %then %do;
    
    /*in case where retail and corporate exposures have different approaches,
      re-classified retail exposure should get treated as corporate exposure   */
   
         data &RESLIB1..retail_granularity_exception ; 
         set &RESLIB1..retail_granularity_exception ; 
             APPROACH = put(APPROACH_CLASS, $MAP_APPROACH.);
            if APPROACH in ('IRB', 'IRBF', 'IRBA') then do;
           if COUNTERPARTY_CLASS = 'RETAIL' then APPROACH = 'IRB';
           /*- non-retail either IRBF or IRBA: if IRB then IRBF -*/
              else if APPROACH = 'IRB' then APPROACH = 'IRBF';
         end;
         run; 
    
      proc sort data = &RESLIB1..retail_granularity_exception ;
            by EXP_ID ;
      run ;
      
        data &RESLIB1..all_exposures; 
        merge &RESLIB1..all_exposures(in=exp)
              &RESLIB1..retail_granularity_exception(keep=EXP_ID CPTY_GROUP_ID COUNTERPARTY_CLASS COUNTERPARTY_SUBCLASS
                                                          IRB_PRODUCT_CLASS STD_RW_BUCKET APPROACH STD_RW_BUCKET_COUNTERPARTY
                                                          IN_DEFAULT_FLG CPTY_CLASS_CHANGED_FLG NUM_DEFAULTED_EXP_CPTY_STD 
                                                          APPROACH_CLASS in=retail)
              ;
        by EXP_ID;
        run;
     
       /* update rw bucket for all exposures to a defaulted corporate counterparty */
        %r00_post_mtm_3_crd4 ;
     
     
       /* update counterparty rw bucket */      
        proc sort data=&RESLIB1..retail_granularity_exception (where=(CPTY_CLASS_CHANGED_FLG=1))
                  out=cpty_retail_1 (keep=COUNTERPARTY_RK EXT_MAP_RATING COUNTERPARTY_CLASS COUNTERPARTY_SUBCLASS) nodupkey;
        by COUNTERPARTY_RK;
        run;
        
        data &RESLIB1..counterparty ; 
        merge &RESLIB1..counterparty (in=a)
              cpty_retail_1(in=b rename=(COUNTERPARTY_RK=CPTY_RK COUNTERPARTY_CLASS=CPTY_CLASS
                                         COUNTERPARTY_SUBCLASS=CPTY_SUBCLASS));
        by CPTY_RK;
        if a;
        
        if b then 
           STD_RW_BUCKET_CPTY = put(EXT_MAP_RATING, $MAP_STD_RW_BUCKET_CORPORATE.);
        run;
     
  /*- 5: reporting -*/
          %put ;
          %print_msg(warning,2,50);
          %print_msg(warning,2,51);
          %print_msg(warning,2,52);
          %put ;
     
    %end;

/*************************************************************************/
/*- Capital deduction qualifying criteria for regulatory SME portfolio  -*/
/***********************************************************************-*/
   /*-  get all non-sec-underlying sme exposures -*/
    data  _all_sme_(keep = EXP_ID EAD COUNTERPARTY_RK COUNTERPARTY_GROUP COUNTERPARTY_CLASS COUNTERPARTY_SUBCLASS
                           STD_PRODUCT_CLASS STD_PRODUCT_SUBCLASS IRB_PRODUCT_CLASS IRB_PRODUCT_SUBCLASS 
                           SME_SUPPORTING_FACTOR_FLG CPTY_GROUP_ID
                    index=(CPTY_GROUP_ID)
                   );
       length CPTY_GROUP_ID $32. ;                  
       set &RESLIB1..all_exposures ;
       where COUNTERPARTY_CLASS in ('RETAIL', 'CORPORATE') and COUNTERPARTY_SUBCLASS in ('SME') and SEC_UNDERLYING_EXP_FLG ne 1 ;
              
       /*- SME capital deduction qualification check is based on obligor client or group of connected clients: CRR Article 501(c) -*/
       if compress(COUNTERPARTY_GROUP) ne '' then CPTY_GROUP_ID = COUNTERPARTY_GROUP;
       else CPTY_GROUP_ID= trim(left(put(COUNTERPARTY_RK, best.))) ;    
    run ;
    
    data _sme_cpty_ ;
       set  _all_sme_ ;
       by CPTY_GROUP_ID ;
       
       retain TOTAL_SME_CPTY 0 ;

       if first.CPTY_GROUP_ID then do;
           TOTAL_SME_CPTY = 0;
       end;
    
       /*- total SME exposure for all qualifying SMEs in a CPTY_GROUP_ID: -*/
       /*- claims secured by residential property are excluded from the total: CRR Article 501(c) -*/
       if not ((STD_PRODUCT_CLASS in ('MORTGAGE', 'LEASE_IMM_PROPERTY') and STD_PRODUCT_SUBCLASS = 'RESIDENTIAL' ) or
               (IRB_PRODUCT_CLASS = 'MORTGAGE' and IRB_PRODUCT_SUBCLASS = 'RESIDENTIAL' ))
        
        then  TOTAL_SME_CPTY = sum(TOTAL_SME_CPTY, EAD);
        

       if last.CPTY_GROUP_ID then output ;
    run ;
    
    data  &RESLIB1..all_sme ;
       merge _all_sme_   _sme_cpty_(keep = CPTY_GROUP_ID TOTAL_SME_CPTY) ;
       by CPTY_GROUP_ID ;
   
      /*- SME portfolio capital deduction qualifying criteria check -*/ 
        if SME_SUPPORTING_FACTOR_FLG = 1 and TOTAL_SME_CPTY <= &SME_CPTY_LIMIT then SME_CAP_DEDUCTION_TRT_FLG = 1 ;
        else SME_CAP_DEDUCTION_TRT_FLG = 0 ;
    run;
    
    
    %local NSME;
        %let NSME = %attrn(&RESLIB1..all_sme, NOBS);
    %if &NSME > 0 %then %do;
    
       proc sort data= &RESLIB1..all_sme ;
              by EXP_ID ;
       run ;

       data &RESLIB1..all_exposures;
           merge &RESLIB1..all_exposures(in=exp) 
                 &RESLIB1..all_sme(keep=EXP_ID SME_CAP_DEDUCTION_TRT_FLG in=sme)
                 ;
           by EXP_ID;
       run;
 
    %end ;

      /* check for the PD LGD components of the changed class*/
      data &RESLIB1..all_exposures ; 
      set &RESLIB1..all_exposures ; 
      length REASON $256.;
          ORIGINAL_APPROACH = APPROACH;
          REASON = '';
          INVALID_COMPONENT_FLG = 0;
          
         /* this needs to be checked only for the CPTY_CLASS_CHANGED_FLG set to 1*/
      if  CPTY_CLASS_CHANGED_FLG = 1 then do; 

          CPTY_PD_FLG = input(put(trim(left(CPTY_PD_MODEL_ID))!!'_'!!trim(left(CPTY_PD_RATING_GRADE)), $MAP_VALID_PD.), 1.);
          PD_FLG = input(put(trim(left(PD_MODEL_ID))!!'_'!!trim(left(PD_MODEL_RATING_GRADE)) , $MAP_VALID_PD.), 1.);
          LGD_FLG = input(put(trim(left(LGD_MODEL_ID))!!'_'!!trim(left(LGD_MODEL_RATING_GRADE)) , $MAP_VALID_LGD.), 1.);
      
          if CPTY_PD_PCT = . and PD_PCT = . and CPTY_PD_FLG = 0 and PD_FLG = 0 then
              PD_FLG = 0;
          else
              PD_FLG = 1;
          /*- check for inputs for IRBA approach -*/
          if APPROACH in ('IRB', 'IRBA') then do;
              /*- PD estimate not available -*/
              if PD_FLG = 0 then do;
                  REASON = "%print_msg(,2,250,1)";
                  INVALID_COMPONENT_FLG = 1;
              end;
              /*- PD estimate available but LGD estimate is not: use IRBF, however if RETAIL, use STD -*/
              else if (LGD_PCT = .) and LGD_FLG = 0 then do;
                  REASON = "%print_msg(,2,251,1)";
                  INVALID_COMPONENT_FLG = 1;
              end;
          end;
          /*- check for inputs for IRBF approach -*/
          else if APPROACH = 'IRBF' then do;
              /*- case where PD estimate not available: -*/
              if PD_FLG = 0 then do;
                  REASON = "%print_msg(,2,250,1)";
                  INVALID_COMPONENT_FLG = 1;
               end;   
          end;
          
        end ;   /*for CPTY_CLASS_CHANGED_FLG =1 */
      run ;
      /* if PD/LGD values are not available then invalidate the exposures */
      data &RESLIB1..all_exposures 
            &RESLIB1..reclass_exp_invalid_components (keep =EXP_ID APPROACH REASON)  ;
      set &RESLIB1..all_exposures ; 
      if INVALID_COMPONENT_FLG then output  &RESLIB1..reclass_exp_invalid_components ;
         else output &RESLIB1..all_exposures ; 
      run; 

   /*Remove the COUNTERPARTY_RK column from retail_granularity_exception table, as does not add much value */
    data &RESLIB1..retail_granularity_exception (drop=COUNTERPARTY_RK)  ;
         set &RESLIB1..retail_granularity_exception ; 
   run ; 
   
    %EXIT :   

%mend r00_post_mtm_6_crd4;
