/*****************************************************************************
 * Copyright (c) 2012 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.
 *
 *    NAME: a00_init_options.sas
 *
 * PURPOSE: Creates macro variables for global parameters such as
 *          reporting currency and rating agencies of choice.
 *
 *   USAGE: 
 *
 *   NOTES: Modified for stored procedures - V4.2
 *          
 *
 *****************************************************************************/
%macro a00_init_options;
/*Clear work library*/
proc datasets library=work nolist kill;
run;quit;
/*Define a table called WORK.PREV_MACROS this will hold the last sessions macros*/
data PREV_macros;
	attrib macro_name length =$32.;
	attrib macro_value length=$32000.;
	stop;
run;
%if (%dsexist(RD_ROOT.gmacros)) EQ 1 %then %do;
	proc append data=rd_root.gmacros base=prev_macros force;
	run;
%end;

/*In case project info table is already present then rename it*/
%if (%dsexist(RD_ROOT.project_info)) EQ 1 %then %do;
	proc datasets library=rd_root nolist;
		change project_info=project_inf1;
	run;
%end;
/*Save the current session macros in RD_ROOT.GMACROS*/
%u00_save_macro_definitions;
/*Move RD_ROOT.GMACROS to WORK.EMP_MACROS*/
proc copy in=rd_root out=work;
	select gmacros;
run;quit;
/*Rename WORK.GMACROS to TEMP_MACROS*/
proc datasets nolist library=work;
	change gmacros=temp_macros;
run;quit;
/*Reset the project info and delete gmacros*/
proc datasets library=rd_root nolist;
	delete project_info;
	change project_inf1=project_info;
	delete gmacros;
run;quit;


/*Put all the options together in one table*/
data options_table;
	attrib macro_name length =$32.;
	attrib macro_value length=$32000.;
	set rd_root.calculation_options(in=incalc) 
		rd_root.output_options(in=inoutp rename=config_value=value) 
		rd_root.crm_allocation_options(in=incrm);
	if incalc or incrm then do;
		if CONFIG_NAME="MAPPING_DATETIME" and CONFIG_VALUE='' then CONFIG_VALUE=dhms(today(),0,0,time());
	end;
	if inoutp then do;
		if CONFIG_NAME="REPORTING_LANGUAGE" then CONFIG_VALUE=klowcase(ktrim(kleft(VALUE)));
		else CONFG_VALUE=ktrim(kleft(VALUE));
		if CONFIG_NAME="MAPPING_DATETIME" and VALUE='' then CONFIG_VALUE=dhms(today(),0,0,time());
		else CONFIG_VALUE=VALUE;
	end;
	MACRO_NAME=CONFIG_NAME;
	macro_value=CONFIG_VALUE;
	keep MACRO_NAME macro_value;
run;
proc sort data=options_table nodupkey;
	by MACRO_NAME macro_value;
	where macro_value is not missing;
run;

/*Define a final macro setting table
For backend run:
The prioritization will be as follows:
1. Consider the latest options from options tables(CALCULATION, OUTPUT AND CRM ALLOCATION OPTIONS)
2. Consider the macros defined in the session which could include macros provided by UI (for example CONFIG_SET)
2. Consider the macro variables from the previous session(GMACROS).

For stored process run:
The prioritization will be as follows:
1. Consider the macros defined in the session which could include macros provided by UI (for example CONFIG_SET)
2. Consider the latest options from options tables(CALCULATION, OUTPUT AND CRM ALLOCATION OPTIONS)
3. Consider the macro variables from the previous session(GMACROS).

*/
%if "&criskm_stpserver"="Y" %then %do;
data all_session_macros;
	merge 
    prev_macros(WHERE=(MACRO_NAME NOT IN ("RPT_LANG0","REPORTING_LANGUAGE0")))
	options_table temp_macros;
	by macro_name;
	call symputx(trim(left(MACRO_NAME)), ktrim(kleft(MACRO_VALUE)),'g');
run;
%end;
%else %do;
data all_session_macros;
	merge 
    prev_macros(WHERE=(MACRO_NAME NOT IN ("RPT_LANG0","REPORTING_LANGUAGE0")))
	temp_macros options_table;
	by macro_name;
	call symputx(trim(left(MACRO_NAME)), ktrim(kleft(MACRO_VALUE)),'g');
run;
%end;

/*Check for Rundate*/
	%global RUNDATE RUNDATETIME  ;

	data _null_;
		set RD_MART.rundate;
		call symput('RUNDATE', trim(left(datepart(RUNDATETIME))));
		call symput('RUNDATETIME', trim(left(RUNDATETIME)));
	run;

	/*- convert to what RD likes -*/
	%let RUNDATE = "%sysfunc(putn(&RUNDATE, DATE9.))"d;

/*----If this is the initial run and all the variables were never defined---*/
%if &frm_a00_init=1 %then %do;
	/*- RUNDATE and RUNDATETIME master username datamart SYSTEM_ORG_RK global macros -*/
	%global RUNDATE RUNDATETIME master username datamart SYSTEM_ORG_RK ;

	data _null_;
		set RD_MART.rundate;
		call symput('RUNDATE', trim(left(datepart(RUNDATETIME))));
		call symput('RUNDATETIME', trim(left(RUNDATETIME)));
	run;

	/*- convert to what RD likes -*/
	%let RUNDATE = "%sysfunc(putn(&RUNDATE, DATE9.))"d;
	
	/*- check a few critical macros -*/
	%if %symglobl(RUNDATE) = 0 %then %do;
	  %let SYSTEM_FATAL_ERROR = Y;
	  %print_msg(error, 2, 155);
	  %print_msg(error, 2, 153);
	%end;
	%else %if %symglobl(CONFIG_SET) = 0 %then %do;
	  %let SYSTEM_FATAL_ERROR = Y;
	  %print_msg(error, 2, 156);
	  %print_msg(error, 2, 153);
	%end;

	/*- check if reporting language has the corresponding nls data set -*/
	%if %dsexist(rd_nls.dataset_lang_&REPORTING_LANGUAGE)=0 %then %do;
	  %print_msg(note,2,125);
	  %let REPORTING_LANGUAGE=en;
	%end;

	/*get value of system_org_rk and configuration_id for further processing*/
	data _null_;
		set rd_stat.entity_system_id (where=(lowcase(SYSTEM_ID) = lowcase("&SYSTEM_ID")));
		call symput ("SYSTEM_ORG_RK", INTERNAL_ORG_RK);
	run;
%end;

proc sort data=sashelp.vmacro out=macros;
    by scope name offset;
run;

*** transpose by scope and name on offset.;
proc transpose data=macros
        out=macros prefix=offset_;
    by scope name;
    id offset;
    var value;
run;

*** reassemble macros to original (wider) values.;
data macros(rename=value2=value);
    format scope name $32. value2 $32000.;
    set macros(  where=(SCOPE='GLOBAL' and  NAME not in: ('MSG')));;
    value2 = cat(of offset_:);
    drop  offset_: _name_ _label_;
run;
data RD_ROOT.gmacros(keep=NAME VALUE rename=(NAME=MACRO_NAME VALUE=MACRO_VALUE));
set macros;
run;

%mend a00_init_options;
