/*****************************************************************************
 * Copyright (c) 2012 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.
 *
 *    NAME: b10_process_market_pd_lgd.sas
 *
 * PURPOSE: Creates current market data for PD and LGD 
 *
 *   USAGE: 
 *
 *   NOTES: Updated to allow users to specify expected loss (EL) models
 *          
 *
 *****************************************************************************/
%macro b10_process_market_pd_lgd;
%global PD_MODEL_USED LGD_MODEL_USED EL_MODEL_USED;
%let PD_MODEL_USED=Y;
%let LGD_MODEL_USED=Y;
%let EL_MODEL_USED=Y;

/*- PD Processing -*/
/*- get current quotes -*/
proc sql;
    create table RD_MKT.current_pd as
        select MODEL_ID, RATING_GRADE, RATING_GRADE_AVG_VALUE
        from RD_STAGE.rating_grade
        where MODEL_TYPE='PD';
quit;

/*- make up names -*/
data RD_MKT.current_pd(keep=PD_NAME SOURCE PD_VALUE label = 'Current PD Rating Grade Values') 
     
    /*- for creating reference variables later -*/
     RD_TEMP.rf_pd_temp(keep=PD_NAME SOURCE MODEL_ID RATING_GRADE rename=(PD_NAME=NAME));

    length PD_NAME SOURCE $32.;
    
    set RD_MKT.current_pd(in=m) RD_CONF.regulatory_pd(in=r);

    if m then do;
        SOURCE='MODEL';
        PD_NAME = 'PD_' !! trim(left(MODEL_ID)) !! '_' !!  trim(left(RATING_GRADE)) ;
    
        /*- convert PD to percentages -*/
        PD_VALUE  = RATING_GRADE_AVG_VALUE * 100 ;
    end;
    
    else do;
        SOURCE = 'REGULATORY';
    end;

    output RD_MKT.current_pd;
    output RD_TEMP.rf_pd_temp;
run;

proc transpose data = RD_MKT.current_pd out = RD_MKT.mkt_pd(drop = _NAME_);
    id PD_NAME ;
    var PD_VALUE;
run;

/*- LGD Processing -*/
/*- get current values from models -*/
proc sql;
    create table RD_MKT.current_lgd as
        select MODEL_ID, RATING_GRADE, RATING_GRADE_AVG_VALUE
        from RD_STAGE.rating_grade
        where MODEL_TYPE='LGD';
quit;

/*- make up names -*/
data RD_MKT.current_lgd(keep=LGD_NAME SOURCE LGD_VALUE label = 'Current LGD Values') 
     
    /*- for creating reference variables later -*/
    RD_TEMP.rf_lgd_temp(keep=LGD_NAME SOURCE MODEL_ID RATING_GRADE rename=(LGD_NAME=NAME));

    length LGD_NAME SOURCE $40.;

    set RD_MKT.current_lgd(in=m) RD_CONF.regulatory_lgd(in=r) ;

    if m then do;
        SOURCE = 'MODEL'; 
        LGD_NAME  = 'LGD_' !! trim(left(MODEL_ID)) !! '_' !!  trim(left(RATING_GRADE)) ;
    
        /*- convert LGD to percentages -*/
        LGD_VALUE = RATING_GRADE_AVG_VALUE * 100 ;
    end;

    else do;
        SOURCE = 'REGULATORY';
    end;

    output RD_MKT.current_lgd;
    output RD_TEMP.rf_lgd_temp;
run;

proc transpose data = RD_MKT.current_lgd out = RD_MKT.mkt_lgd(drop = _NAME_);
    id LGD_NAME;
    var LGD_VALUE;
run;

/*- EL Processing */
/*- Extract values for current EL models */
proc sql ;
    create table RD_MKT.current_el as
        select MODEL_ID, RATING_GRADE, RATING_GRADE_AVG_VALUE
        from RD_STAGE.rating_grade
        where MODEL_TYPE = 'EL' ;
quit ;

/*- Generate names for risk factor variables for average values -*/
data RD_MKT.current_el (keep = EL_NAME EL_VALUE label = 'Current EL Values')
    
    /*- A second data set for creating reference variable keys -*/
    RD_TEMP.rf_el_temp (keep = EL_NAME MODEL_ID RATING_GRADE rename = (EL_NAME=NAME)) ;
    
    length EL_NAME $32. ;
    set RD_MKT.current_el ;
    
    EL_NAME = 'EL_'!!trim(left(MODEL_ID))!!'_'!!trim(left(RATING_GRADE)) ;
    
    /*- Convert to a percentage -*/
    EL_VALUE = RATING_GRADE_AVG_VALUE*100 ;
    
    output RD_MKT.current_el ;
    output RD_TEMP.rf_el_temp ;
    
run ;

/* Don't use models for EL if none are defined */
%if %attrn(RD_TEMP.rf_el_temp, nobs)=0 %then %let EL_MODEL_USED=N;

/*- To be used as a current data set, the average EL values must be transposed -*/
proc transpose data = RD_MKT.current_el out = RD_MKT.mkt_el(drop = _NAME_) ;
    id EL_NAME ;
    var EL_VALUE ;
run ;


/*- create validation format -*/
data fmt (keep=FMTNAME START END LABEL TYPE HLO);
set RD_TEMP.rf_pd_temp (where=(not missing(MODEL_ID) or not missing(RATING_GRADE))) end=lastobs;
      FMTNAME            = 'MAP_VALID_PD' ;
      START              = trim(left(MODEL_ID)) !! '_' !!  trim(left(RATING_GRADE))  ;
      END                = START ;
      LABEL              = 1;
      TYPE               = 'C' ;
      output ;
      if lastobs then do;
         START           ='**OTHER**' ;
         END             = START;
         HLO='O' ;
         LABEL=0;
         output ;
      end;
run;

%if %attrn(fmt, nobs)=0 %then %do;
    data fmt;
        FMTNAME = 'MAP_VALID_PD' ;
        START   ='**OTHER**' ;
        END     = START;
        HLO='O' ;
        LABEL=0;
        type='C';
        output ;
    run;

    %let PD_MODEL_USED=N;
%end;

proc format cntlin=fmt lib=RD_ENV; run;

data fmt (keep=FMTNAME START END LABEL TYPE HLO);
set RD_TEMP.rf_lgd_temp (where=(not missing(MODEL_ID) or not missing(RATING_GRADE))) end=lastobs;
      FMTNAME            = 'MAP_VALID_LGD' ;
      START              = trim(left(MODEL_ID)) !! '_' !!  trim(left(RATING_GRADE)) ;
      END                = START ;
      LABEL              = 1;
      TYPE               = 'C' ;
      output ;
      if lastobs then do;
         START           ='**OTHER**' ;
         END             = START;
         HLO='O' ;
         LABEL=0;
         output ;
      end;
run;

%if %attrn(fmt, nobs)=0 %then %do;
    data fmt;
        FMTNAME = 'MAP_VALID_LGD' ;
        START   ='**OTHER**' ;
        END     = START;
        HLO='O' ;
        LABEL=0;
        type='C';
        output ;
    run;

    %let LGD_MODEL_USED=N;
%end;

proc format cntlin=fmt lib=RD_ENV; 
run;

%mend b10_process_market_pd_lgd;
