/*****************************************************************************
 * Copyright (c) 2015 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.
 *
 *    NAME: create_dummy_buckets.sas
 *
 * PURPOSE: Creates dummy risk weight buckets to avoid unresolved reference 
 *          variables related warnings in the log.
 *
 *   USAGE: %create_dummy_buckets ;
 *          
 *
 *   NOTES: 
 *
 *
 *****************************************************************************/

%macro create_dummy_buckets ;

	
	proc sort data = RD_TEMP.ref_rf_rw out = work.ref_rf_rw_dummy_buckets nodup ;
	by refvar refkey ; 
	run ; 
	
	/*Remove Basel reference from the dataset
	  and extract the bucket value */ 
	
	data work.ref_rf_rw_dummy_buckets ; 
	set work.ref_rf_rw_dummy_buckets ; 
	if refvar = 'BASELI_RW_REF' then delete ;
	BucketNo = compress(refkey, ' ', 'A');
	BucketNo1 = input(BucketNo,3.); 
	drop BucketNo; 
	rename BucketNo1=BucketNo; 
	run ;
	
	proc sort data = work.ref_rf_rw_dummy_buckets; 
	by refvar BucketNo ; 
	run; 
	data work.ref_rf_rw_dummy_buckets ;
	set work.ref_rf_rw_dummy_buckets ;
	by refvar ;
	if last.refvar then output ; 
	run; 
	
	/* Get the highest bucket no available*/
	proc sql noprint;
	select max(BucketNo) into :Highest_bucket_no
	from work.ref_rf_rw_dummy_buckets;
	quit;
	
	/*get the total no of reference variable for which*/
	/*dummy buckets are required to be created*/
	%let total_ref_vars = %attrn(work.ref_rf_rw_dummy_buckets, NLOBS);
	
	/*Store ref vars in the macro variables*/
	data _null_;
	set work.ref_rf_rw_dummy_buckets end=eof;
	retain cnt 0;
	cnt+1;
	call symput("Ref_var_"||left(cnt),refvar);
	if eof then call symput("refvars",left(cnt));
	run;
	
	/*This loop will create the dummy buckets for the each reference variable*/
	%do j = 1 %to &total_ref_vars ; 
	
		data work.dummy_buckets_&j ; 
		set work.ref_rf_rw_dummy_buckets ;
		where REFVAR = "&&Ref_var_&j."; ; 
		run; 
	
		data _null_ ; 
		set work.dummy_buckets_&j ; 
		call symput('Start_of_Bucket', BucketNo);
		call symput('Refvar_value', REFVAR);
		call symput('Name_value', Name);
		run; 
	
		data work.dummy_buckets_&j (keep = NAME REFVAR REFKEY) ;; 
		length NAME REFVAR REFKEY REFKEY $32.; 
		%if &Start_of_Bucket < &Highest_bucket_no %then %do ; 
			%do i = &Start_of_Bucket + 1  %to &Highest_bucket_no ;
			   NAME = TRANWRD("&Name_value",compress("&Start_of_Bucket."),"&i") ; 
			   REFVAR = "&Refvar_value" ; 
			   REFKEY = "BUCKET&i";  
			   output;
			%end; 
		%end;
	    stop ; 
		run; 
	%end; 
	
	/* Create a single dataset consisting of all the dummy buckets */
	data rd_temp.Ref_rf_rw_dummy  ; 
	  set %do jj = 1 %to &total_ref_vars ; 
	          work.dummy_buckets_&jj
		  %end;
	      ; 
	run ; 
	
	/* append the generated dummy bucket data into Ref_rf_rw dataset*/
	proc append base= RD_TEMP.ref_rf_rw data =  rd_temp.Ref_rf_rw_dummy ; 
	
	/* create the risk weight values */ 
			data work.dummy_rw_data ; 
			set rd_temp.Ref_rf_rw_dummy ; 
			rename NAME = RW_NAME ;
			RW_VALUE = 0;
			run ;
			
			proc transpose data = work.dummy_rw_data 
			                      out = work.mkt_rw_dummy(drop = _NAME_);
			id RW_NAME;
			var RW_VALUE;
			run;
			
	/* Append the dummy risk weights into mkt_current dataset */		
			data rd_mkt.mkt_current ; 
			set rd_mkt.mkt_current work.mkt_rw_dummy ; 
			run ; 
			
	
	/* Create a dummy risk factor variables dataset corresponding 
	   to dummy reference variables */
	data RD_TEMP.var_rf_rw_dummy (keep = KIND NAME LABEL ROLE TYPE GROUP MLEVEL);
	
	    length KIND     $8. ;
	    length NAME     $32.;
	    length LABEL    $40.;
	    length ROLE     $8. ;
	    length TYPE     $8. ;
	    length GROUP    $32.;
	    length MLEVEL   $8.;
	
	    set work.dummy_rw_data ; 
	
	    NAME = upcase(RW_NAME) ;
	    LABEL= NAME;
	    KIND = 'FACTOR';
	    TYPE = 'NUM' ;
	    ROLE = 'VAR';
	    MLEVEL = 'INTERVAL';
	    GROUP = 'Dummy Risk Weights';
	
	run;
	
	
	/* Dummy risk factor registration in RiskDimensions		*/
	proc risk noseqcheck ;
		environment open = RD_TEMP.&RD_TEMPLATE ;
		/*- data driven rfvars registration -*/
		readvars data = RD_TEMP.var_rf_rw_dummy;
		environment save noupdatemetadata ;
	
	run;		

	/*Deleting temporary dummy_bucket datasets*/
	proc datasets lib=work memtype=data nolist;
		delete dummy_buckets_: ;
	quit;
%mend create_dummy_buckets; 
 
