/*****************************************************************************
 * Copyright (c) 2012 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.
 *
 *    NAME: enrich_exposures_rec.sas
 *
 * PURPOSE: Stores code for enriching purchased receivables exposures.
 *
 *   USAGE: %enrich_exposures_rec(posds, issueds, outposds);
 *          posds    : (pos) input position data set
 *          issueds  : (pos) input issue data set
 *          outposds : (pos) output position data set
 *
 *   NOTES: 
 *
 *
 *****************************************************************************/

%macro enrich_exposures_rec(posds, issueds, outposds);

/*- enrich data with issue attributes and ratings -*/
    proc sort data = &posds out = &outposds ;
    by ISSUE_KEY;
    run;

    proc sort data = &issueds ;
    by ISSUE_KEY;
    run;

    data &outposds;
    merge &outposds(in = a) RD_POS.issue_ratings RD_POS.issue_int_ratings
          &issueds(rename=(RECEIVABLE_POOL_RK=ISSUE_RECEIVABLE_POOL_RK));
    by ISSUE_KEY;
    if a;
    run;

/*- enrich data with counterparty dimensional data -*/
    proc sort data = &outposds;
    by COUNTERPARTY_RK;
    run;

    data &outposds;
    merge &outposds(in=a) 
          RD_POS.counterparty_dim
          ;
    by COUNTERPARTY_RK;
    if a;
    run;

/*- enrich data with internal organization attributes -*/
    proc sort data = &outposds;
    by INTERNAL_ORG_RK;
    run;

    data &outposds(drop = INVALID_APPROACH_FLG %if &NUM_AGENCIES_ISSUE>0 %then %do; EXT_ISSUE_RATING_: 
                          EXT_ISSUE_MAP_RATING_: CRM_ADJUSTED_FLG_: SHORT_TERM_ASSESS_FLG_: %end;  
                          INT_ISSUE_RATING ISSUE_RECEIVABLE_POOL_RK LOCATION ORIGINAL_APPROACH REASON
                          INVALID_COMPONENT_FLG CPTY_PD_FLG PD_FLG LGD_FLG ELIGIBLE_DD_TREAT_GUARANTOR
                          BASELI_CCY_FLAG BASELI_ST_TYPE BASELI_PRODUCT_CLASS BASELI_PRODUCT_SUBCLASS 
                          BASELI_PRODUCT
                   index=(RECEIVABLE_POOL_RK)
                  )
         &outposds._sec(drop = INVALID_APPROACH_FLG LOCATION ORIGINAL_APPROACH REASON INVALID_COMPONENT_FLG
                               CPTY_PD_FLG PD_FLG LGD_FLG ELIGIBLE_DD_TREAT_GUARANTOR
                        index=(RECEIVABLE_POOL_RK))
         invalid_approach(keep = INSTID COUNTERPARTY_CLASS STD_PRODUCT_CLASS STD_PRODUCT_SUBCLASS IRB_PRODUCT_CLASS IRB_PRODUCT_SUBCLASS
                   APPROACH_CLASS APPROACH )
         invalid_components (keep=INSTID LOCATION ORIGINAL_APPROACH APPROACH REASON CPTY_PD_MODEL_ID CPTY_PD_RATING_GRADE PD_MODEL_ID 
                                  PD_MODEL_RATING_GRADE LGD_MODEL_ID LGD_MODEL_RATING_GRADE PD_PCT LGD_PCT CPTY_PD_PCT)
         ; 

    merge &outposds(in =a) RD_STAGE.internal_org_mart(in = b);
    by INTERNAL_ORG_RK;
    if a;
  
    length STD_PRODUCT_CLASS STD_PRODUCT_SUBCLASS IRB_PRODUCT_CLASS IRB_PRODUCT_SUBCLASS POSITION_CATEGORY
           APPROACH_CLASS INSTTYPE OFF_BALANCE_SHEET_TYPE CCF_TYPE HAIRCUT_SET HAIRCUT_TYPE CCF_SET
           CCF_SET_ID BASELI_RW_CLASS LOCATION REASON $32. 
           ORIGINAL_APPROACH APPROACH BASELI_CCY_FLAG $16.;
    length EXT_ISSUE_MAP_RATING HAIRCUT_RATING_BUCKET HAIRCUT_RES_MAT_BAND STD_RW_BUCKET STD_RW_BUCKET_SHORT BASELI_ST_TYPE            
           $8.;

/*- check on currency -*/
    if  CURRENCY = '' then CURRENCY="&REPORT_CCY" ;

/*- get seniority flag -*/
    %get_seniority_flg;    

/*- flag exposures underlying securitization pools -*/ 
    %get_securitized_exposure_flg;

    RECEIVABLE_EXPOSURE_FLG = 1;

/*- determine if the exposure is a pool -*/
    if RECEIVABLE_POOL_RK = ISSUE_RECEIVABLE_POOL_RK then 
        RECEIVABLE_POOL_FLG = 1;
    else 
        RECEIVABLE_POOL_FLG = 0;

/*- if exposures come in as pools, get their counterparty classes -*/
/*- needed for assigning appropriate pricing methods              -*/
    if RECEIVABLE_POOL_FLG = 1 or COUNTERPARTY_CLASS = '' then do;
        COUNTERPARTY_CLASS = put(REGULATORY_POOL_CPTY_TYPE_CD, $MAP_COUNTERPARTY_CLASS.);
        COUNTERPARTY_SUBCLASS = put(REGULATORY_POOL_CPTY_TYPE_CD, $MAP_COUNTERPARTY_SUBCLASS.);
    end;

/*- get original and residual maturity -*/
    %get_maturity(INCEPTION_DATE, MATURITY_DATE);

/*- get product class and subclass -*/
    %get_product_class;

/*- get regulatory approach -*/
    %get_regulatory_approach;

/*- check proper approach assignment -*/
    %check_asset_class_approach;
    
    if INVALID_APPROACH_FLG EQ 1 then 
       output invalid_approach;
/*- process the following only for valid approaches -*/
    else do;   
    
/*- check necessary component for IRB aproaches -*/
       %check_component;   
       LOCATION="&outposds";
       if INVALID_COMPONENT_FLG = 1 then
          output invalid_components;    
          
/*- determine if it is a short-term claim: original maturity of 3 months or less -*/
    %get_shortterm_claim_flg;    
    
/*- get instrument type for MtM pricing -*/
       %get_mtm_insttype;

/*- get off-balance-sheet type -*/
       %get_obsi_type;

/*- get mapped issue rating: either from second best or from specified rating agency -*/
       %get_mapped_ratings(RATING_TYPE=ISSUE);
       INT_ISSUE_MAP_RATING = put(INT_ISSUE_RATING, $INT.);
       
/*- get risk weight buckets -*/
       %get_std_rw_buckets;
       
/*- get Basel I risk weight mapping -*/
       %get_basel_i_rw;
   
/*- get haircut info -*/
       %get_haircut_info;

/*- get ccf info -*/
       if (IRB_PRODUCT_CLASS= 'PURCHASED_REC' and IRB_PRODUCT_SUBCLASS='REVOLVING') then do;
       /*- no own estimate is allowed -*/
           CCF_SET_ID = "&CCF_SET";
       /*- ccf type is not mapped from off-balance-sheet type code -*/
           CCF_TYPE   = "CCF_REC_REVOLVING";         

           CCF_SET = put(CCF_SET_ID, $MAP_CCF_SET.);
       end;
/*- get position type -*/
       %get_position_type;

       COLLATERAL = 0;
 
/*- direct to the output data set -*/
       if SEC_UNDERLYING_EXP_FLG EQ 1 then 
         output &outposds._sec;
       else 
         output &outposds;
    end;
    
/*- append invalid approach data set -*/
    %if %attrn(invalid_approach, NOBS) GT 0 %then %do;
       proc append base=RD_POS.invalid_approach data=invalid_approach;
       run;
    %end;
    
/*- append invalid component data set -*/
    %if %attrn(invalid_components, NOBS) GT 0 %then %do;
       proc append base=RD_POS.all_exp_with_invalid_components data=invalid_components force;
       run;
    %end;
    
/*- get pool characteristics for underlying receivables -*/
    data _pool_(index=(RECEIVABLE_POOL_RK));
    set &issueds (where = (RECEIVABLE_POOL_RK NE .) 
                  keep = RECEIVABLE_POOL_RK DILUTION_EXPECTED_LOSS_PERCENT
                 )
    ;
    run;

    data &outposds;
    merge &outposds(in=pos drop=DILUTION_EXPECTED_LOSS_PERCENT) _pool_;
    by RECEIVABLE_POOL_RK;
    if pos;
    run;

    data &outposds._sec;
    merge &outposds._sec(in=pos drop=DILUTION_EXPECTED_LOSS_PERCENT) _pool_;
    by RECEIVABLE_POOL_RK;
    if pos;
    run;
    
/* Redirect the rows which doesn't have valid Instrument Type */     
   data &outposds	 invalid_insttypes ;
	     set &outposds; 
	     if INVALID_INSTTYPE_FLG eq 1 then output invalid_insttypes ;
	     else output 
	         &outposds; 
	    run ;  

   data &outposds._sec invalid_insttypes_sec ; 
     set &outposds._sec; 
		 if INVALID_INSTTYPE_FLG eq 1 then output invalid_insttypes_sec ;
	     else output 
	        &outposds._sec; 
    run ;  	    
/*- append invalid Instrument Type data set -*/
    %if %attrn(invalid_insttypes, NOBS) GT 0 %then %do;
       proc append base=RD_POS.all_exp_invalid_insttypes data=invalid_insttypes force;
       run;
    %end;

/*- append invalid Instrument sec Type data set -*/
    %if %attrn(invalid_insttypes_sec, NOBS) GT 0 %then %do;
       proc append base=RD_POS.all_exp_invalid_insttypes_sec data=invalid_insttypes_sec force;
       run;
    %end;
	    
    
%mend enrich_exposures_rec;
