/*****************************************************************************
 * Copyright (c) 2014 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.
 *
 *    NAME: get_cva_risk_amts_crd4.sas
 *
 * PURPOSE: Code to calculate CVA risk amounts for CVA under standardized     
 *          approach and alternative approach.
 *                   
 *   USAGE: %get_cva_risk_amts_crd4
 *               CRMAPP  : CRM approach applied (optimal or by_rank)
 *               ALTCALC : If alternative approach should be applied to OEM exposures (Y/N)
 *
 *           
 *   NOTES: 
 *          
 *****************************************************************************/
%macro get_cva_risk_amts_crd4(CRMAPP=,ALTCALC=);

/*- 1. get cva exposure info -*/
/*- get all derivative and repo_style exposures subject to CVA capital charge -*/
data _cva_exp_all_ (index=(NETTING_SET_RK) drop=CRM_TYPE) ;
   set &RESLIB1..crm_allocation_&CRMAPP._detail
       (keep=NETTING_SET_RK EXP_ID COUNTERPARTY_RK CRM_TYPE EAD EAD_ADJ CVA_STD_MAT_EFF CVA_STD_WGT 
             STD_PRODUCT_CLASS IRB_PRODUCT_CLASS PE_APPROACH 
             COUNTERPARTY_CLASS COUNTERPARTY_SUBCLASS CCP_CPTY_SUBCLASS EEA_MEMBER_FLG CENTGOV_TRT_FLG RW_EXP              
        );
   if (IRB_PRODUCT_CLASS in ('DERIVATIVE','REPO_STYLE') or STD_PRODUCT_CLASS in ('DERIVATIVE','REPO_STYLE')) and missing(CRM_TYPE) ;
   
run ;


%if %attrn(_cva_exp_all_,nobs) > 0 %then %do;

   /*- for each netting set, keep the netted exposure and remove original exposures forming the netting_set from the list of CVA exposures -*/
   data _cva_exp_list_ ;
      set _cva_exp_all_ ;
       by NETTING_SET_RK ;

      if not missing(NETTING_SET_RK) then do ;
         if index(EXP_ID, 'MNA_OBSI_') > 0 or index(EXP_ID, 'MNA_REPO_') > 0 then output ;
      end ;

      else output ;
   run ;

   /*- get CVA_EAD -*/
   data _cva_exp_list_ (index=(COUNTERPARTY_RK));
      set _cva_exp_list_ ;
      
    /*- define CVA_EAD for each EXP_ID -*/
      if not missing(EAD_ADJ) then CVA_EAD = EAD_ADJ ;          
      else  CVA_EAD = EAD ;

      if STD_PRODUCT_CLASS = 'REPO_STYLE' or IRB_PRODUCT_CLASS = 'REPO_STYLE' then
         CVA_EAD_SFT = CVA_EAD ; 
      else 
         CVA_EAD_OTC = CVA_EAD ; 
   run ;
   
   /*- get total CVA_EAD for non-financial counterparties.                     -*/
   /*- TOT_CVA_EAD is used to compare with clearing threshold to determine     -*/
   /*- if the counterparty should be included for CVA capital charge.          -*/
   proc summary data=_cva_exp_list_ ;
      var CVA_EAD ;
      by COUNTERPARTY_RK ;
      where COUNTERPARTY_CLASS ne 'INSTITUTION' and COUNTERPARTY_SUBCLASS not in ('FINANCIAL_ENTITY','INSURANCE','NQCCP') ;
      output out=thresh sum=TOT_CVA_EAD ;
   run ;   
   
   /*- separate exposures subject to CVA capital charge into two datasets for   -*/
   /*- standardized and alternative CVA calculation respectively.               -*/
   data _cva_exp_list_ (keep=COUNTERPARTY_RK CVA_EAD CVA_EAD_SFT CVA_EAD_OTC CVA_STD_MAT_EFF CVA_STD_WGT)
        _cva_alt_input_ (keep=EXP_ID COUNTERPARTY_RK CVA_EAD CVA_EAD_SFT CVA_EAD_OTC index=(EXP_ID));
      merge _cva_exp_list_ thresh(keep=COUNTERPARTY_RK TOT_CVA_EAD);
      by COUNTERPARTY_RK ;

    /*- Counterparty exclusion: CRR Article 382                       -*/
    /*- exclude transactions cleared through QCCP: CRR Article 382(3) -*/
      if (COUNTERPARTY_CLASS = 'INSTITUTION') then do ;
         if (COUNTERPARTY_SUBCLASS = 'QCCP' or CCP_CPTY_SUBCLASS ='QCCP') then delete ;
      end ;
    
    /*- exclude exposures to central banks of EU countries: CRR Article 382(4)(d) -*/ 
      else if (COUNTERPARTY_CLASS = 'SOVEREIGN' and COUNTERPARTY_SUBCLASS = 'CENTRAL_BANK' and EEA_MEMBER_FLG = 'YES') then delete ; 
 
    /*- exclude exposures to highly rated MDB: CRR Article 382(4)(d) -*/
      else if (COUNTERPARTY_CLASS = 'SOVEREIGN' and COUNTERPARTY_SUBCLASS = 'HR_MDB') then delete ; 
 
    /*- exclude exposures to PSE treated as central government: CRR Article 382(4)(d) -*/
      else if (COUNTERPARTY_SUBCLASS = 'PSE' and CENTGOV_TRT_FLG = 1) then delete ;
    
    /*- exclude exposures to regional/local government that have zero risk weight: CRR Article 382(4)(d) -*/  
      else if (COUNTERPARTY_SUBCLASS in('REGIONAL_GOV','LOCAL_GOV') and RW_EXP = 0) then delete ;

    /*- exclude non-financials whose total CVA exposures do not exceed clearing threshold: CRR Article 382(4)(a) -*/
      else do ;     
         if (COUNTERPARTY_CLASS ne 'INSTITUTION') and (COUNTERPARTY_SUBCLASS not in ('FINANCIAL_ENTITY','INSURANCE','NQCCP'))
            and (TOT_CVA_EAD <= &CLEARING_THRESHOLD ) then delete ;
      end ;  


      %if &ALTCALC = Y %then %do ; 
         if PE_APPROACH ='OE' then
            output _cva_alt_input_; 
         else output _cva_exp_list_ ; 
      %end ; 
      %else %do ; 
         output _cva_exp_list_ ;
      %end ; 
   run ;

   %if %attrn(_cva_exp_list_,nobs) > 0 %then %do; 
   
      /********* standardized CVA capital charge calculation **********/

      /* get total CVA_EAD and effective maturity for a counterparty */
      data _cva_cpty_calc_ (keep = COUNTERPARTY_RK CVA_STD_WGT CVA_EAD_TOTAL CVA_EAD_SFT_TOTAL CVA_EAD_OTC_TOTAL CVA_CPTY_STD_MAT);
         set _cva_exp_list_ ;
         retain CVA_EAD_TOTAL CVA_EAD_SFT_TOTAL CVA_EAD_OTC_TOTAL _wgt_mat ;
         by COUNTERPARTY_RK ;

         if first.COUNTERPARTY_RK then do ;
            CVA_EAD_TOTAL = 0 ;
            CVA_EAD_SFT_TOTAL = 0 ;
            CVA_EAD_OTC_TOTAL = 0 ;
            _wgt_mat =0 ;
         end;   

         /*- get total CVA_EAD -*/
         CVA_EAD_TOTAL = sum(CVA_EAD_TOTAL, CVA_EAD) ;
         CVA_EAD_SFT_TOTAL = sum(CVA_EAD_SFT_TOTAL, CVA_EAD_SFT);
         CVA_EAD_OTC_TOTAL = sum(CVA_EAD_OTC_TOTAL, CVA_EAD_OTC );

         /*- get EAD-weighted average of effective maturity for CVA exposures with a counterparty -*/
         _wgt_mat = sum(_wgt_mat, CVA_STD_MAT_EFF * CVA_EAD);

         if last.COUNTERPARTY_RK then do ;
            if(CVA_EAD_TOTAL > 0) then CVA_CPTY_STD_MAT = _wgt_mat / CVA_EAD_TOTAL ;
            else CVA_CPTY_STD_MAT = 0 ;
            output ;
         end ;
      run ;   

      /*- 2. get cds info -*/
      /*- get all CDS used for CVA risk hedging -*/
      data _cva_cds_list_ ; 
         set "&REGPATH&PATHSEP.all_crms"
             (keep=REFERENCE_CPTY_RK CRM_ISSUE_KEY CVA_HEDGE_FLG CRM_VALUE CRM_RESIDUAL_MATURITY INDEX_CDS_CVA_STD_RSK_WGT_PCT CRM_TYPE CRM_SUBTYPE
              rename=(CRM_ISSUE_KEY = ISSUE_KEY CRM_RESIDUAL_MATURITY=RESIDUAL_MATURITY));

         if CRM_TYPE = 'CREDIT_DERIVATIVE' and CVA_HEDGE_FLG = 1 ;
      run ;

      /*- single name and index CDS are separated into 2 datasets */
      data _cva_cds_single_name_(index=(REFERENCE_CPTY_RK))  
           _cva_cds_index_ (index=(ISSUE_KEY));
         set _cva_cds_list_ ;

         if not missing(REFERENCE_CPTY_RK) then output _cva_cds_single_name_ ;
         else if CRM_SUBTYPE = 'INDEX' then output _cva_cds_index_ ;
      run ;


      /*- single name CDS for CVA hedging */
      data _cva_sn_cds_calc_ (keep=REFERENCE_CPTY_RK CVA_SN_CDS_NOTL_TOTAL CVA_SN_CDS_MAT );
         set _cva_cds_single_name_ ;
         by REFERENCE_CPTY_RK ;
         retain CVA_SN_CDS_NOTL_TOTAL _wgt_mat ;

         if first.REFERENCE_CPTY_RK then do ;
            CVA_SN_CDS_NOTL_TOTAL = 0 ;
            _wgt_mat = 0 ;
         end ;

         /*- get total notional amount of single name CDS for a reference counterparty -*/
         CVA_SN_CDS_NOTL_TOTAL = CVA_SN_CDS_NOTL_TOTAL + CRM_VALUE ;
         _wgt_mat =  sum(_wgt_mat, RESIDUAL_MATURITY * CRM_VALUE);

         if last.REFERENCE_CPTY_RK then do ;
            /*- get notional-weighted average of residual maturity for all  -*/
            /*- single named CDS of a reference counterparty                -*/
            if CVA_SN_CDS_NOTL_TOTAL > 0 then 
               CVA_SN_CDS_MAT = _wgt_mat / CVA_SN_CDS_NOTL_TOTAL ;         
            else CVA_SN_CDS_MAT = 0 ;   
            output ;
         end ;
      run ;   


      /*- index CDS for CVA hedging -*/
      data _cva_indx_cds_calc_ (drop=_wgt_mat);
         set _cva_cds_index_ ;
         retain CVA_INDEX_CDS_NOTL_TOTAL  _wgt_mat ;
         by ISSUE_KEY ;

         if first.ISSUE_KEY then do ;
            CVA_INDEX_CDS_NOTL_TOTAL = 0 ;
            _wgt_mat = 0 ;
         end ;

         /*- get total notional amount of an index CDS used for CVA hedging -*/
         CVA_INDEX_CDS_NOTL_TOTAL = CVA_INDEX_CDS_NOTL_TOTAL + CRM_VALUE ;
         _wgt_mat =  sum(_wgt_mat, RESIDUAL_MATURITY * CRM_VALUE);

         if last.ISSUE_KEY then do ;

            /*- get notional-weighted average of residual maturity for an -*/
            /*- index CDS used for CVA hedging. -*/
            if CVA_INDEX_CDS_NOTL_TOTAL >0 then do ;
               CVA_INDEX_CDS_MAT = _wgt_mat / CVA_INDEX_CDS_NOTL_TOTAL ;

               /*- discount factor applied to total notional amount of an index CDS -*/
               NOTL_df = (1-exp(-&CVA_STD_DISC_FACTOR_MULT * CVA_INDEX_CDS_MAT))/(&CVA_STD_DISC_FACTOR_MULT * CVA_INDEX_CDS_MAT); 

               CVA_INDEX_CDS_HEDGE = INDEX_CDS_CVA_STD_RSK_WGT_PCT / 100 * CVA_INDEX_CDS_MAT * CVA_INDEX_CDS_NOTL_TOTAL * NOTL_df ;  

               output ;
            end ;

         end ;
      run ;

      /*- sum up over all index CDS -*/
      proc summary data = _cva_indx_cds_calc_ ;
         var CVA_INDEX_CDS_HEDGE CVA_INDEX_CDS_NOTL_TOTAL;
         output out= _cva_indx_cds_tot_ 
                sum= CVA_INDEX_CDS_HEDGE_SUM CVA_INDEX_NOTIONAL_AMT ;
      run ;


      /*-3. calculate the standardized CVA capital charge -*/
      data _cva_cpty_calc_ ;
         merge _cva_cpty_calc_ 
               _cva_sn_cds_calc_ (rename=(REFERENCE_CPTY_RK=COUNTERPARTY_RK));
            by COUNTERPARTY_RK ;

            if  CVA_CPTY_STD_MAT >0 then 
            EAD_df = (1-exp(-&CVA_STD_DISC_FACTOR_MULT * CVA_CPTY_STD_MAT))/(&CVA_STD_DISC_FACTOR_MULT * CVA_CPTY_STD_MAT);
            if  CVA_SN_CDS_MAT >0 then
            NOTL_df = (1-exp(-&CVA_STD_DISC_FACTOR_MULT * CVA_SN_CDS_MAT))/(&CVA_STD_DISC_FACTOR_MULT * CVA_SN_CDS_MAT);

            /*- assume no overhedging of any single name CDS, CVA_STD_CPTY_EAD_ADJ cannot be negative -*/ 
            CVA_STD_CPTY_EAD_ADJ = max(0,sum(CVA_CPTY_STD_MAT * CVA_EAD_TOTAL * EAD_df, -CVA_SN_CDS_MAT * CVA_SN_CDS_NOTL_TOTAL * NOTL_df)) ;
            CVA_STD_CPTY_VALUE = &CVA_STD_CORR_FACTOR * CVA_STD_WGT / 100 * CVA_STD_CPTY_EAD_ADJ ;
            CVA_STD_CPTY_VALUE_SQ = (1-&CVA_STD_CORR_FACTOR**2) * (CVA_STD_WGT / 100 * CVA_STD_CPTY_EAD_ADJ)**2 ; 

      run ;

      /*- sum up over all counterparty */
      proc summary data = _cva_cpty_calc_ ;
         var CVA_STD_CPTY_VALUE CVA_STD_CPTY_VALUE_SQ CVA_EAD_TOTAL CVA_EAD_SFT_TOTAL CVA_EAD_OTC_TOTAL CVA_SN_CDS_NOTL_TOTAL ;
         output out = _cva_cpty_tot_ 
                sum = CVA_STD_CPTY_VALUE_SUM CVA_STD_CPTY_VALUE_SQ_SUM EAD_ADJ CVA_EAD_ADJ_SFT CVA_EAD_ADJ_OTC CVA_CDS_NOTIONAL_AMT 
                  N = CVA_COUNTERPARTY_COUNT;
      run ;

      /*- get standardized CVA capital charge amount -*/
      data _cva_calc_final_ ;
         merge _cva_cpty_tot_ (drop = _TYPE_ _FREQ_ )
               _cva_indx_cds_tot_ (keep= CVA_INDEX_CDS_HEDGE_SUM CVA_INDEX_NOTIONAL_AMT) ;

         CVA_CHARGE_AMT =&CVA_STD_RSK_CHG_MULTIPLIER * sqrt(&CVA_RISK_HORIZON)* sqrt((CVA_STD_CPTY_VALUE_SUM-CVA_INDEX_CDS_HEDGE_SUM)**2 + CVA_STD_CPTY_VALUE_SQ_SUM);  
         CVA_CALCULATION_METHOD_TYPE = 'STD' ;
         drop CVA_STD_CPTY_VALUE_SUM CVA_STD_CPTY_VALUE_SQ_SUM CVA_INDEX_CDS_HEDGE_SUM ; 
      run ;


      /********* alternative CVA capital charge calculation **********/

      /*-4. If applicable, produce the alternative measure for exposures utilizing -*/
      /*-   the OEM approach (CRR Article 385).                                    -*/
      %if %attrn(_cva_alt_input_, NOBS) %then %do;
         data _cva_alt_rwa_ ;
            set "&OUTPATH.&PATHSEP.capital_detail_&CRMAPP."(keep=EXP_ID RWA) ;
         run ;
         proc sort data=_cva_alt_rwa_ ;
            by EXP_ID;
         run;

         data _cva_alt_input_ ;
            merge _cva_alt_input_ (in=a)
                  _cva_alt_rwa_ ;
               by EXP_ID ;
               if a ;

            CVA_ALT_CHARGE_AMT = &CVA_ALT_MULTIPLICATION_FCTR * RWA * &CAPITAL_RATIO ;

         run ;

         proc sort data=_cva_alt_input_ ;
            by COUNTERPARTY_RK ; 
         run ;

         proc summary data=_cva_alt_input_ ;
            var CVA_EAD CVA_EAD_SFT CVA_EAD_OTC CVA_ALT_CHARGE_AMT ;
            by COUNTERPARTY_RK ;
            output out=_cva_alt_res_ sum= CVA_EAD CVA_EAD_SFT CVA_EAD_OTC CVA_ALT_CHARGE_AMT;
         run ;

         proc summary data=_cva_alt_res_ ;
            var CVA_EAD CVA_EAD_SFT CVA_EAD_OTC CVA_ALT_CHARGE_AMT ;
            output out= _cva_alt_final_ 
                   sum= EAD_ADJ CVA_EAD_ADJ_SFT CVA_EAD_ADJ_OTC CVA_CHARGE_AMT 
                     n= CVA_COUNTERPARTY_COUNT;
         run ;

         data _cva_alt_final_ ;
            set _cva_alt_final_ (keep= EAD_ADJ CVA_EAD_ADJ_SFT CVA_EAD_ADJ_OTC CVA_CHARGE_AMT CVA_COUNTERPARTY_COUNT);

            CVA_CALCULATION_METHOD_TYPE = 'ALT' ;
         run ;

         /*- output CVA results to be appended to the detail table -*/
         proc append base= _cva_calc_final_ data= _cva_alt_final_  force nowarn ;
         run ;
      %end ;

      proc append base= &RESLIB1..cva_capital_charge data= _cva_calc_final_  force nowarn ;
      run ;
      
   %end ;
%end ;

%mend get_cva_risk_amts_crd4 ;
