/*****************************************************************************
 * Copyright (c) 2012 by SAS Institute Inc., Cary, NC, USA. All Rights Reserved.
 *
 *    NAME: r00_post_mtm_4.sas
 *
 * PURPOSE: Process repo-style transactions subject to master netting agreements.
 *          
 *
 *   USAGE: 
 *
 *   NOTES: 1. Repo-style transactions need to have both exposure and crm sides.
 *          2. Transactions subject to the same netting agreement must be subject
 *             to the same regulatory approach, i.e. STD/IRBF/IRBA.
 *          3. All netting agreements here are bilateral, otherwise they are not
 *             recognized.
 *
 *
 *****************************************************************************/


%macro r00_post_mtm_4;

%global N_REPO_NET ;

/*- 1: get repo-style transaction exposures, CRMs, and netting set information -*/
/*- get all repo-style transactions subject to MNAs -*/
    data _all_exp_repo_(index=(NETTING_SET_RK));
    set &RESLIB1..all_exposures;
    
    /*- get the notional amount for weighting the maturity of a repo_style exposure -*/
    FACE_VALUE_AMT = coalesce(FACE_VALUE_AMT, VALUE_AMT, BALANCE_AMT, EXP_VALUE);

    where (STD_PRODUCT_CLASS = 'REPO_STYLE' or IRB_PRODUCT_CLASS = 'REPO_STYLE') 
          and NETTING_SET_RK NE .;
    run;

/*- if no such exposures, exit -*/
    %local N_REPOS;
    %let N_REPOS = %attrn(_all_exp_repo_, NOBS);
    %if &N_REPOS = 0 %then %do ;
        %let N_REPO_NET = 0 ;
        %goto EXIT;
    %end ;
    
/*- get notional amount weighted average maturity, inception date and maturity date for a netting set -*/
    proc summary data=_all_exp_repo_(keep=NETTING_SET_RK FACE_VALUE_AMT REPO_INCEPTION_DT REPO_MATURITY_DT RESIDUAL_MAT_REPO_CONTRACT);
    var REPO_INCEPTION_DT REPO_MATURITY_DT RESIDUAL_MAT_REPO_CONTRACT ;
    weight FACE_VALUE_AMT;
    by NETTING_SET_RK;
    output out=_all_repo_mat_(keep=NETTING_SET_RK REPO_INCEPTION_DT REPO_MATURITY_DT RESIDUAL_MAT_REPO_CONTRACT) 
           mean=REPO_INCEPTION_DT REPO_MATURITY_DT RESIDUAL_MAT_REPO_CONTRACT ;
    run;

    data _all_repo_mat_;
     set _all_repo_mat_;
        MAT_EFF =  min(&IRBA_M_CAP, max(&IRBA_M_FLOOR_REPO_NETTING, RESIDUAL_MAT_REPO_CONTRACT)); 
        CVA_STD_MAT_EFF = max(&IRBA_M_FLOOR_REPO_NETTING, RESIDUAL_MAT_REPO_CONTRACT);
    run;
        
/*- get all CRMs related to repo-style transactions -*/
    proc sql;
    create table _all_crm_repo_(index=(NETTING_SET_RK)) as
    select exp.NETTING_SET_RK, crm.ISSUE_KEY, crm.HAIRCUT as CRM_HAIRCUT, crm.CRM_VALUE, crm.HAIRCUT_HOLDING_DAYS,
           crm.HAIRCUT_SCALE, crm.CCY as CRM_CCY
    from &RESLIB1..all_crms as crm 
    inner join RD_POS.exposure_crm_link as link 
    on crm.CRM_ID = link.CRM_ID
    inner join _all_exp_repo_ as exp 
    on exp.EXP_ID = link.EXP_ID;
    quit;

/*- all repo-style exposures and CRMs subject to MNAs -*/    
    data _all_repo_;
    set _all_exp_repo_(in=exp keep=NETTING_SET_RK ISSUE_KEY HAIRCUT CCY FACE_VALUE_AMT EXP_VALUE)
        _all_crm_repo_(in=crm rename=(CRM_CCY=CCY))
        ;
    by NETTING_SET_RK;
    run;  

/*- 2: get total exposure, total CRMs, and netted position in each security and currency -*/
/*- get netting set information -*/
    data _all_repo_(drop=EST_HOLDING_DAYS 
                    index=(SET_SEC=(NETTING_SET_RK ISSUE_KEY) SET_FX=(NETTING_SET_RK CCY)));
    merge _all_repo_(in=repo)
          RD_POS.netting_set
          (in=net_set keep=NETTING_SET_RK NETTING_SETTLEMENT_CURRENCY);
    by NETTING_SET_RK;
    if repo and net_set;

    if CCY NE NETTING_SETTLEMENT_CURRENCY then do;
        HAIRCUT_FX = input(put(compress(CCY||":"||NETTING_SETTLEMENT_CURRENCY), MAP_HAIRCUT_FX.), best12.);
        if not missing(HAIRCUT_SCALE) then do;
           EST_HOLDING_DAYS = 
             input(put(compress(CCY||":"||NETTING_SETTLEMENT_CURRENCY), MAP_H_FX_HOLDING_DAYS.), best12.);
           if (not missing(HAIRCUT_HOLDING_DAYS)) and (not missing(EST_HOLDING_DAYS)) then  
             HAIRCUT_FX = HAIRCUT_FX * HAIRCUT_SCALE * sqrt ( HAIRCUT_HOLDING_DAYS / EST_HOLDING_DAYS );
        end;
    end;
    else
        HAIRCUT_FX = 0;
    run;

/*- get net position in each security -*/
    data _all_repo_sec_(keep=NETTING_SET_RK SUM_FACE SUM_E SUM_C SUM_SEC CRM_TYPE);
	length CRM_TYPE $32;
    set _all_repo_(keep=NETTING_SET_RK ISSUE_KEY FACE_VALUE_AMT EXP_VALUE CRM_VALUE HAIRCUT CRM_HAIRCUT); 
    by NETTING_SET_RK ISSUE_KEY;
    retain SUM_FACE SUM_E SUM_C SUM_SEC NET_SEC 0;
    
    if first.NETTING_SET_RK then do;
        SUM_FACE = 0;
        SUM_E   = 0;
        SUM_C   = 0;  
        SUM_SEC = 0;
    end;

    SUM_FACE = sum(SUM_FACE, FACE_VALUE_AMT); 
   /*adding the following 2 lines      */
    SUM_E = sum(SUM_E, EXP_VALUE); 
    SUM_C = sum(SUM_C, CRM_VALUE); 
    if first.ISSUE_KEY then do;
        NET_SEC = 0;
    end;

    /*- absolute value of the netted position in a given security -*/
    NET_SEC = sum(EXP_VALUE*sum(0, HAIRCUT)/100, CRM_VALUE*sum(0, CRM_HAIRCUT)/100);

    if last.ISSUE_KEY then do;
        NET_SEC = abs(NET_SEC);
        SUM_SEC = sum(SUM_SEC, NET_SEC);
        CRM_TYPE = 'FINANCIAL_COLLATERAL' ;
    end;
       
    if last.NETTING_SET_RK then output;

    run;

/*- get net position in each currency -*/
    data _all_repo_fx_(keep=NETTING_SET_RK SUM_FX);
    set _all_repo_(keep=NETTING_SET_RK CCY EXP_VALUE CRM_VALUE HAIRCUT_FX);
    by NETTING_SET_RK CCY;
    retain SUM_FX NET_FX 0;
    
    if first.NETTING_SET_RK then do;
        SUM_FX = 0;
    end;

    if first.CCY then do;
        NET_FX = 0;
    end;

    /*- absolute value of the netted position in a given currency -*/
    NET_FX = sum(EXP_VALUE*sum(0, HAIRCUT_FX)/100, CRM_VALUE*sum(0, HAIRCUT_FX)/100);

    if last.CCY then do;
        NET_FX = abs(NET_FX);
        SUM_FX = sum(SUM_FX, NET_FX);
    end;

    if last.NETTING_SET_RK then output;

    run;

/*- 3: create a netted position for each netting set -*/
/*- and inherit other useful information, especially counterparty info -*/
    data &RESLIB1..all_exposures_repo_net
         (drop=SUM_: NETTING_SET_INCEPTION_DATE NETTING_SET_MATURITY_DATE index=(NETTING_SET_RK));
    merge _all_exp_repo_(in=exp_repo)
          RD_POS.netting_set
          (keep=NETTING_SET_RK NETTING_SETTLEMENT_CURRENCY VAR_RT INCEPTION_DATE MATURITY_DATE
           rename=(NETTING_SETTLEMENT_CURRENCY=CCY  
                   INCEPTION_DATE = NETTING_SET_INCEPTION_DATE MATURITY_DATE=NETTING_SET_MATURITY_DATE
                  )
           in=net_set
          )
          _all_repo_mat_(in=wa_mat) 
          _all_repo_sec_(in=pos_sec)
          _all_repo_fx_(in=pos_fx)
          ;
    by NETTING_SET_RK;

    if exp_repo and net_set and wa_mat and pos_sec and pos_fx;

    /*- only need info from the first exposure -*/
    if first.NETTING_SET_RK then do;
    
        /*- create EXP_ID for the netted position -*/
        EXP_ID = compress('MNA_REPO_' || put(NETTING_SET_RK, 16.));    
        
        /*- user may want to use netting set maturity info if not missing -*/
        if NETTING_SET_MATURITY_DATE ne . and NETTING_SET_INCEPTION_DATE ne . then do;
           %get_maturity(NETTING_SET_INCEPTION_DATE, NETTING_SET_MATURITY_DATE);
            MAT_EFF =  min(&IRBA_M_CAP, max(&IRBA_M_FLOOR_REPO_NETTING, RESIDUAL_MATURITY));
            CVA_STD_MAT_EFF = max(&IRBA_M_FLOOR_REPO_NETTING, RESIDUAL_MATURITY); 
        end;   
        else do;
           %get_maturity(REPO_INCEPTION_DT, REPO_MATURITY_DT);
        end;
  
        /*- update values -*/
        /*- face value -*/
        FACE_VALUE_AMT = SUM_FACE;

        /*- total exposure value -*/
        EXP_VALUE = SUM_E;
        EAD = SUM_E ; 
        
        /*- VaR approach -*/
        if VAR_RT NE . then do;
           EAD_ADJ = max(0, sum(SUM_E, -SUM_C)*sum(1, VAR_RT/100));
        end;
        /*- haircut approach -*/
        else do;
           EAD_ADJ = max(0, sum(SUM_E, -SUM_C, SUM_SEC, SUM_FX));  
        end;
      
       /*Adjust the values for further processing */
        EXP_VALUE = EAD_ADJ;
        EAD = EAD_ADJ ;  
        output;
 
    end;  
    
    run;

    %let N_REPO_NET = %attrn(&RESLIB1..all_exposures_repo_net, NOBS);

    %if &N_REPO_NET > 0 %then %do;
/*- 4: append these netted positions to exposures -*/
/*- original exposures and crms are also kept for later processing -*/
/*- to see the effect of netting -*/ 
/*- enrich the repo positions if "novation" position exist -*/
        %if &NETEXPDATA EQ YES > 0 %then %do;
        proc sort data=RD_netting_exposures;
        by ORIG_NETTING_SET_RK;
        run;
        
        data &RESLIB1..all_exposures_repo_net;
        merge &RESLIB1..all_exposures_repo_net (in=a)
              "&NETEXPPATH.&pathsep.instvals"(drop=ORIG_NETTING_SET_RK VALUE MARGIN_PERIOD_DAYS_CNT 
                                                   MARGIN_AGREEMENT_THRESHOLD_AMT COLLATERAL SHORTPOSITION
                                                   HOLDING FACE_VALUE_AMT MAX_ORIGINAL_MATURITY OUTV_EAD
                                                   OUTV_MAT_EFF OUTV_EXPOSURE_PROVISION OUTV_GENERAL_PROVISION
                                                   OUTV_CRM: OUTV_HAIRCUT_SCALE INSTSOURCE INSTNUMBER INSTTYPE 
                                                   OUTV_ADD_ON CURRENCY BASEDATE MAX_RESIDUAL_MATURITY INSTID
                                                   RESIDUAL_MATURITY ORIGINAL_MATURITY FACE_VALUE_AMT
                                              rename =(OUTV_HAIRCUT=HAIRCUT 
                                                       OUTV_CCF=CCF OUTV_RESIDUAL_VALUE=RESIDUAL_VALUE)
                                              in=b) 
        by NETTING_SET_RK;
        if a and b;
        run;
        %end;

/*- 4: append these netted positions to exposures -*/
/*- original exposures are also kept for later processing -*/
/*- to see the effect of netting -*/    
    data  &reslib1..all_exp_removed_repo_net_pos;
     length NOTE $100. ;
                set _all_exp_repo_;
                NOTE="These exposures belongs to a CCR netting agreement and are summed up in a new CCR netting instrument";
    run;

    proc sql;
         update  &RESLIB1..All_exposures set EAD=0 where exp_id in (select exp_id from &RESLIB1..All_exp_removed_repo_net_pos);
         update  &RESLIB1..All_exposures set EXP_VALUE=0 where exp_id in (select exp_id from &RESLIB1..All_exp_removed_repo_net_pos);
    quit;
/* Add a column EAD_ADJ to the &RESLIB1..all_exposures table.              */
/* Note that only the netted repo cases with EXP_ID MNA_.. have non-missing  */
/* values for EAD_ADJ in table &RESLIB1..all_exposures.                    */
/* Add CRM_TYPE for the netted repo */
        proc sql ;
         alter table &RESLIB1..all_exposures
         add EAD_ADJ num label='EAD_ADJ', CRM_TYPE char(32) format $32. label ='CRM_TYPE';
        quit;
        
        proc append base=&RESLIB1..all_exposures data=&RESLIB1..all_exposures_repo_net force; run;
    
/*- report -*/
        %put ;
        %print_msg(note,2,45);
        %print_msg(note,2,46);
        %print_msg(note,2,47);
        %put ;
    %end;

    %EXIT:

    %clean_lib; 
    
%mend r00_post_mtm_4;
